@@ -285,6 +285,27 @@ fn deimplify(rt: &mut ReturnType) {
     }
 }
 
+/// Remove any generics that place constraints on Self.
+fn dewhereselfify(generics: &mut Generics) {
+    if let Some(ref mut wc) = &mut generics.where_clause {
+        let new_predicates = wc.predicates.iter()
+            .filter(|wp| match wp {
+                WherePredicate::Type(pt) => {
+                    pt.bounded_ty != parse2(quote!(Self)).unwrap()
+                },
+                _ => true
+            }).cloned()
+            .collect::<Punctuated<WherePredicate, Token![,]>>();
+        wc.predicates = new_predicates;
+    }
+    if generics.where_clause.as_ref()
+        .map(|wc| wc.predicates.is_empty())
+        .unwrap_or(false)
+    {
+        generics.where_clause = None;
+    }
+}
+
 /// Remove any mutability qualifiers from a method's argument list
 fn demutify(inputs: &mut Punctuated<FnArg, token::Comma>) {
     for arg in inputs.iter_mut() {
@@ -447,7 +468,7 @@ fn deselfify_args(
 {
     for arg in args.iter_mut() {
         if let FnArg::Typed(pt) = arg {
-            deselfify(&mut *pt.ty, actual, generics)
+            deselfify(pt.ty.as_mut(), actual, generics)
         }
     }
 }
@@ -1381,6 +1402,46 @@ mod deselfify {
     }
 }
 
+mod dewhereselfify {
+    use super::*;
+
+    #[test]
+    fn lifetime() {
+        let mut meth: ImplItemMethod = parse2(quote!(
+                fn foo<'a>(&self) where 'a: 'static, Self: Sized;
+        )).unwrap();
+        let expected: ImplItemMethod = parse2(quote!(
+                fn foo<'a>(&self) where 'a: 'static;
+        )).unwrap();
+        dewhereselfify(&mut meth.sig.generics);
+        assert_eq!(meth, expected);
+    }
+
+    #[test]
+    fn normal_method() {
+        let mut meth: ImplItemMethod = parse2(quote!(
+                fn foo(&self) where Self: Sized;
+        )).unwrap();
+        let expected: ImplItemMethod = parse2(quote!(
+                fn foo(&self);
+        )).unwrap();
+        dewhereselfify(&mut meth.sig.generics);
+        assert_eq!(meth, expected);
+    }
+
+    #[test]
+    fn with_real_generics() {
+        let mut meth: ImplItemMethod = parse2(quote!(
+                fn foo<T>(&self, t: T) where Self: Sized, T: Copy;
+        )).unwrap();
+        let expected: ImplItemMethod = parse2(quote!(
+                fn foo<T>(&self, t: T) where T: Copy;
+        )).unwrap();
+        dewhereselfify(&mut meth.sig.generics);
+        assert_eq!(meth, expected);
+    }
+}
+
 mod gen_keyid {
     use super::*;
 
@@ -1439,9 +1500,7 @@ mod merge_generics {
         let wc1: WhereClause = parse2(quote!(where T: Default)).unwrap();
         g1.where_clause = Some(wc1.clone());
 
-        let g2 = g1.clone();
-
-        let gm = super::merge_generics(&g1, &g2);
+        let gm = super::merge_generics(&g1, &g1);
         let gm_wc = &gm.where_clause;
 
         assert_eq!(quote!(#g1 #wc1).to_string(),
