@@ -222,7 +222,7 @@ impl<'a> Builder<'a> {
                 false,
             ),
             ReturnType::Type(_, ref ty) => {
-                let mut output_ty = supersuperfy(&**ty, self.levels);
+                let mut output_ty = supersuperfy(ty, self.levels);
                 destrify(&mut output_ty);
                 let boxed = dedynify(&mut output_ty);
                 (output_ty, boxed)
@@ -269,7 +269,6 @@ impl<'a> Builder<'a> {
         let alifetimes = salifetimes.into_iter()
             .collect::<HashSet<LifetimeDef>>()
             .union(&malifetimes.into_iter().collect::<HashSet<_>>())
-            .into_iter()
             .cloned()
             .collect();
 
@@ -406,9 +405,9 @@ pub(crate) struct MockFunction {
     /// Types used for Predicates.  Will be almost the same as args, but every
     /// type will be a non-reference type.
     predty: Vec<Type>,
-    /// Does the function return a non-'static reference? 
+    /// Does the function return a non-'static reference?
     return_ref: bool,
-    /// Does the function return a mutable reference? 
+    /// Does the function return a mutable reference?
     return_refmut: bool,
     /// References to every type in `predty`.
     refpredty: Vec<Type>,
@@ -448,10 +447,19 @@ impl MockFunction {
         let no_match_msg = quote!(std::format!(
             "{}: No matching expectation found", #desc));
         let sig = &self.sig;
-        let vis = if self.trait_.is_some() {
-            &Visibility::Inherited
+        let (vis, dead_code) = if self.trait_.is_some() {
+            (&Visibility::Inherited, quote!())
         } else {
-            &self.call_vis
+            let dead_code = if let Visibility::Inherited = self.call_vis {
+                // This private method may be a helper only used by the struct's
+                // other methods, which we are mocking.  If so, the mock method
+                // will be dead code.  But we can't simply eliminate it, because
+                // it might also be used by other code in the same module.
+                quote!(#[allow(dead_code)])
+            } else {
+                quote!()
+            };
+            (&self.call_vis, dead_code)
         };
         let substruct_obj = if let Some(trait_) = &self.trait_ {
             let ident = format_ident!("{}_expectations", trait_);
@@ -477,6 +485,7 @@ impl MockFunction {
             quote!(
                 // Don't add a doc string.  The original is included in #attrs
                 #(#attrs)*
+                #dead_code
                 #vis #sig {
                     let no_match_msg = #no_match_msg;
                     #deref {
@@ -497,6 +506,7 @@ impl MockFunction {
             quote!(
                 // Don't add a doc string.  The original is included in #attrs
                 #(#attrs)*
+                #dead_code
                 #vis #sig {
                     let no_match_msg = #no_match_msg;
                     #deref self.#substruct_obj #name.#call#tbf(#(#call_exprs,)*)
@@ -874,7 +884,7 @@ impl<'a> ToTokens for Common<'a> {
                                 m);
                         });
                     self.verify_sequence(desc);
-                    if self.times.is_satisfied() {
+                    if ::mockall::ExpectedCalls::TooFew != self.times.is_satisfied() {
                         self.satisfy_sequence()
                     }
                 }
@@ -952,15 +962,26 @@ impl<'a> ToTokens for Common<'a> {
 
             impl #ig Drop for Common #tg #wc {
                 fn drop(&mut self) {
-                    if !::std::thread::panicking() && !self.times.is_satisfied()
-                    {
+                    if !::std::thread::panicking() {
                         let desc = std::format!(
                             "{}", self.matcher.lock().unwrap());
-                        panic!("{}: Expectation({}) called {} time(s) which is fewer than expected {}",
-                               #funcname,
-                               desc,
-                               self.times.count(),
-                               self.times.minimum());
+                        match self.times.is_satisfied() {
+                            ::mockall::ExpectedCalls::TooFew => {
+                                panic!("{}: Expectation({}) called {} time(s) which is fewer than expected {}",
+                                    #funcname,
+                                    desc,
+                                    self.times.count(),
+                                    self.times.minimum());
+                            },
+                            ::mockall::ExpectedCalls::TooMany => {
+                                panic!("{}: Expectation({}) called {} time(s) which is more than expected {}",
+                                    #funcname,
+                                    desc,
+                                    self.times.count(),
+                                    self.times.maximum());
+                            },
+                            _ => ()
+                        }
                     }
                 }
             }
@@ -1513,9 +1534,7 @@ impl<'a> ToTokens for Context<'a> {
             }
             impl #ty_ig Drop for Context #ty_tg #ty_wc {
                 fn drop(&mut self) {
-                    if !std::thread::panicking() {
-                        Self::do_checkpoint()
-                    }
+                    Self::do_checkpoint()
                 }
             }
         ).to_tokens(tokens);
@@ -1647,13 +1666,13 @@ impl<'a> ToTokens for RefRfunc<'a> {
                 {
                     match self {
                         Rfunc::Default(Some(ref __mockall_o)) => {
-                            Ok(__mockall_o)
+                            ::std::result::Result::Ok(__mockall_o)
                         },
                         Rfunc::Default(None) => {
                             Err(#default_err_msg)
                         },
                         Rfunc::Const(ref __mockall_o) => {
-                            Ok(__mockall_o)
+                            ::std::result::Result::Ok(__mockall_o)
                         },
                         Rfunc::_Phantom(_) => unreachable!()
                     }
@@ -1717,7 +1736,7 @@ impl<'a> ToTokens for RefMutRfunc<'a> {
                 {
                     match self {
                         Rfunc::Default(Some(ref mut __mockall_o)) => {
-                            Ok(__mockall_o)
+                            ::std::result::Result::Ok(__mockall_o)
                         },
                         Rfunc::Default(None) => {
                             Err(#default_err_msg)
@@ -1726,7 +1745,7 @@ impl<'a> ToTokens for RefMutRfunc<'a> {
                         {
                             *__mockall_o = Some(__mockall_f(#(#argnames, )*));
                             if let Some(ref mut __mockall_o2) = __mockall_o {
-                                Ok(__mockall_o2)
+                                ::std::result::Result::Ok(__mockall_o2)
                             } else {
                                 unreachable!()
                             }
@@ -1737,13 +1756,13 @@ impl<'a> ToTokens for RefMutRfunc<'a> {
                                     #(#argnames, )*)
                             );
                             if let Some(ref mut __mockall_o2) = __mockall_o {
-                                Ok(__mockall_o2)
+                                ::std::result::Result::Ok(__mockall_o2)
                             } else {
                                 unreachable!()
                             }
                         },
                         Rfunc::Var(ref mut __mockall_o) => {
-                            Ok(__mockall_o)
+                            ::std::result::Result::Ok(__mockall_o)
                         },
                         Rfunc::_Phantom(_) => unreachable!()
                     }
@@ -1812,15 +1831,15 @@ impl<'a> ToTokens for StaticRfunc<'a> {
                             Err("called twice, but it returns by move")
                         },
                         Rfunc::Mut(__mockall_f) => {
-                            Ok(__mockall_f( #(#argnames, )* ))
+                            ::std::result::Result::Ok(__mockall_f( #(#argnames, )* ))
                         },
                         Rfunc::MutSt(__mockall_f) => {
-                            Ok((__mockall_f.get_mut())(#(#argnames,)*))
+                            ::std::result::Result::Ok((__mockall_f.get_mut())(#(#argnames,)*))
                         },
                         Rfunc::Once(_) => {
                             if let Rfunc::Once(mut __mockall_f) =
                                 mem::replace(self, Rfunc::Expired) {
-                                Ok(__mockall_f( #(#argnames, )* ))
+                                ::std::result::Result::Ok(__mockall_f( #(#argnames, )* ))
                             } else {
                                 unreachable!()
                             }
@@ -1828,7 +1847,7 @@ impl<'a> ToTokens for StaticRfunc<'a> {
                         Rfunc::OnceSt(_) => {
                             if let Rfunc::OnceSt(mut __mockall_f) =
                                 mem::replace(self, Rfunc::Expired) {
-                                Ok((__mockall_f.into_inner())(#(#argnames,)*))
+                                ::std::result::Result::Ok((__mockall_f.into_inner())(#(#argnames,)*))
                             } else {
                                 unreachable!()
                             }
